@@ -5,7 +5,7 @@
  * © 2009-2013 Michael Stapelberg and contributors (see also: LICENSE)
  *
  * i3-nagbar is a utility which displays a nag message, for example in the case
- * when the user has an error in his configuration file.
+ * when the user has an error in their configuration file.
  *
  */
 #include <stdio.h>
@@ -131,7 +131,7 @@ static void handle_button_release(xcb_connection_t *conn, xcb_button_release_eve
     printf("button released on x = %d, y = %d\n",
            event->event_x, event->event_y);
     /* If the user hits the close button, we exit(0) */
-    if (event->event_x >= (rect.width - 32))
+    if (event->event_x >= (rect.width - logical_px(32)))
         exit(0);
     button_t *button = get_button_at(event->event_x, event->event_y);
     if (!button)
@@ -164,7 +164,9 @@ static void handle_button_release(xcb_connection_t *conn, xcb_button_release_eve
     char *link_path;
     char *exe_path = get_exe_path(argv0);
     sasprintf(&link_path, "%s.nagbar_cmd", script_path);
-    symlink(exe_path, link_path);
+    if (symlink(exe_path, link_path) == -1) {
+        err(EXIT_FAILURE, "Failed to symlink %s to %s", link_path, exe_path);
+    }
 
     char *terminal_cmd;
     sasprintf(&terminal_cmd, "i3-sensible-terminal -e %s", link_path);
@@ -188,21 +190,23 @@ static void handle_button_release(xcb_connection_t *conn, xcb_button_release_eve
  */
 static int handle_expose(xcb_connection_t *conn, xcb_expose_event_t *event) {
     /* re-draw the background */
-    xcb_change_gc(conn, pixmap_gc, XCB_GC_FOREGROUND, (uint32_t[]) {color_background});
+    xcb_change_gc(conn, pixmap_gc, XCB_GC_FOREGROUND, (uint32_t[]){color_background});
     xcb_poly_fill_rectangle(conn, pixmap, pixmap_gc, 1, &rect);
 
     /* restore font color */
     set_font_colors(pixmap_gc, color_text, color_background);
     draw_text(prompt, pixmap, pixmap_gc,
-              4 + 4, 4 + 4, rect.width - 4 - 4);
+              logical_px(4) + logical_px(4),
+              logical_px(4) + logical_px(4),
+              rect.width - logical_px(4) - logical_px(4));
 
     /* render close button */
     const char *close_button_label = "X";
-    int line_width = 4;
+    int line_width = logical_px(4);
     /* set width to the width of the label */
     int w = predict_text_width(i3string_from_utf8(close_button_label));
     /* account for left/right padding, which seems to be set to 8px (total) below */
-    w += 8;
+    w += logical_px(8);
     int y = rect.width;
     uint32_t values[3];
     values[0] = color_button_background;
@@ -212,23 +216,25 @@ static int handle_expose(xcb_connection_t *conn, xcb_expose_event_t *event) {
     xcb_rectangle_t close = {y - w - (2 * line_width), 0, w + (2 * line_width), rect.height};
     xcb_poly_fill_rectangle(conn, pixmap, pixmap_gc, 1, &close);
 
-    xcb_change_gc(conn, pixmap_gc, XCB_GC_FOREGROUND, (uint32_t[]) {color_border});
+    xcb_change_gc(conn, pixmap_gc, XCB_GC_FOREGROUND, (uint32_t[]){color_border});
     xcb_point_t points[] = {
         {y - w - (2 * line_width), line_width / 2},
         {y - (line_width / 2), line_width / 2},
-        {y - (line_width / 2), (rect.height - (line_width / 2)) - 2},
-        {y - w - (2 * line_width), (rect.height - (line_width / 2)) - 2},
+        {y - (line_width / 2), (rect.height - (line_width / 2)) - logical_px(2)},
+        {y - w - (2 * line_width), (rect.height - (line_width / 2)) - logical_px(2)},
         {y - w - (2 * line_width), line_width / 2}};
     xcb_poly_line(conn, XCB_COORD_MODE_ORIGIN, pixmap, pixmap_gc, 5, points);
 
     values[0] = 1;
     set_font_colors(pixmap_gc, color_text, color_button_background);
     /* the x term here seems to set left/right padding */
-    draw_text_ascii(close_button_label, pixmap, pixmap_gc, y - w - line_width + w / 2 - 4,
-                    4 + 4 - 1, rect.width - y + w + line_width - w / 2 + 4);
+    draw_text_ascii(close_button_label, pixmap, pixmap_gc,
+                    y - w - line_width + w / 2 - logical_px(4),
+                    logical_px(4) + logical_px(3),
+                    rect.width - y + w + line_width - w / 2 + logical_px(4));
     y -= w;
 
-    y -= 20;
+    y -= logical_px(20);
 
     /* render custom buttons */
     line_width = 1;
@@ -236,21 +242,21 @@ static int handle_expose(xcb_connection_t *conn, xcb_expose_event_t *event) {
         /* set w to the width of the label */
         w = predict_text_width(buttons[c].label);
         /* account for left/right padding, which seems to be set to 12px (total) below */
-        w += 12;
-        y -= 30;
-        xcb_change_gc(conn, pixmap_gc, XCB_GC_FOREGROUND, (uint32_t[]) {color_button_background});
-        close = (xcb_rectangle_t) {y - w - (2 * line_width), 2, w + (2 * line_width), rect.height - 6};
+        w += logical_px(12);
+        y -= logical_px(30);
+        xcb_change_gc(conn, pixmap_gc, XCB_GC_FOREGROUND, (uint32_t[]){color_button_background});
+        close = (xcb_rectangle_t){y - w - (2 * line_width), logical_px(2), w + (2 * line_width), rect.height - logical_px(6)};
         xcb_poly_fill_rectangle(conn, pixmap, pixmap_gc, 1, &close);
 
-        xcb_change_gc(conn, pixmap_gc, XCB_GC_FOREGROUND, (uint32_t[]) {color_border});
+        xcb_change_gc(conn, pixmap_gc, XCB_GC_FOREGROUND, (uint32_t[]){color_border});
         buttons[c].x = y - w - (2 * line_width);
         buttons[c].width = w;
         xcb_point_t points2[] = {
-            {y - w - (2 * line_width), (line_width / 2) + 2},
-            {y - (line_width / 2), (line_width / 2) + 2},
-            {y - (line_width / 2), (rect.height - 4 - (line_width / 2))},
-            {y - w - (2 * line_width), (rect.height - 4 - (line_width / 2))},
-            {y - w - (2 * line_width), (line_width / 2) + 2}};
+            {y - w - (2 * line_width), (line_width / 2) + logical_px(2)},
+            {y - (line_width / 2), (line_width / 2) + logical_px(2)},
+            {y - (line_width / 2), (rect.height - logical_px(4) - (line_width / 2))},
+            {y - w - (2 * line_width), (rect.height - logical_px(4) - (line_width / 2))},
+            {y - w - (2 * line_width), (line_width / 2) + logical_px(2)}};
         xcb_poly_line(conn, XCB_COORD_MODE_ORIGIN, pixmap, pixmap_gc, 5, points2);
 
         values[0] = color_text;
@@ -258,13 +264,15 @@ static int handle_expose(xcb_connection_t *conn, xcb_expose_event_t *event) {
         set_font_colors(pixmap_gc, color_text, color_button_background);
         /* the x term seems to set left/right padding */
         draw_text(buttons[c].label, pixmap, pixmap_gc,
-                  y - w - line_width + 6, 4 + 3, rect.width - y + w + line_width - 6);
+                  y - w - line_width + logical_px(6),
+                  logical_px(4) + logical_px(3),
+                  rect.width - y + w + line_width - logical_px(6));
 
         y -= w;
     }
 
     /* border line at the bottom */
-    line_width = 2;
+    line_width = logical_px(2);
     values[0] = color_border_bottom;
     values[1] = line_width;
     xcb_change_gc(conn, pixmap_gc, XCB_GC_FOREGROUND | XCB_GC_LINE_WIDTH, values);
@@ -316,7 +324,7 @@ int main(int argc, char *argv[]) {
 
     argv0 = argv[0];
 
-    char *pattern = sstrdup("-misc-fixed-medium-r-normal--13-120-75-75-C-70-iso10646-1");
+    char *pattern = sstrdup("pango:monospace 8");
     int o, option_index = 0;
     enum { TYPE_ERROR = 0,
            TYPE_WARNING = 1 } bar_type = TYPE_ERROR;
@@ -408,14 +416,14 @@ int main(int argc, char *argv[]) {
     xcb_create_window(
         conn,
         XCB_COPY_FROM_PARENT,
-        win,                                                 /* the window id */
-        root,                                                /* parent == root */
-        50, 50, 500, font.height + 8 + 8 /* 8 px padding */, /* dimensions */
-        0,                                                   /* x11 border = 0, we draw our own */
+        win,                                                                         /* the window id */
+        root,                                                                        /* parent == root */
+        50, 50, 500, font.height + logical_px(8) + logical_px(8) /* 8 px padding */, /* dimensions */
+        0,                                                                           /* x11 border = 0, we draw our own */
         XCB_WINDOW_CLASS_INPUT_OUTPUT,
         XCB_WINDOW_CLASS_COPY_FROM_PARENT, /* copy visual from parent */
         XCB_CW_BACK_PIXEL | XCB_CW_EVENT_MASK,
-        (uint32_t[]) {
+        (uint32_t[]){
             0, /* back pixel: black */
             XCB_EVENT_MASK_EXPOSURE |
                 XCB_EVENT_MASK_STRUCTURE_NOTIFY |
@@ -465,7 +473,7 @@ int main(int argc, char *argv[]) {
     } __attribute__((__packed__)) strut_partial;
     memset(&strut_partial, 0, sizeof(strut_partial));
 
-    strut_partial.top = font.height + 6;
+    strut_partial.top = font.height + logical_px(6);
     strut_partial.top_start_x = 0;
     strut_partial.top_end_x = 800;
 
@@ -481,7 +489,7 @@ int main(int argc, char *argv[]) {
     /* Create pixmap */
     pixmap = xcb_generate_id(conn);
     pixmap_gc = xcb_generate_id(conn);
-    xcb_create_pixmap(conn, root_screen->root_depth, pixmap, win, 500, font.height + 8);
+    xcb_create_pixmap(conn, root_screen->root_depth, pixmap, win, 500, font.height + logical_px(8));
     xcb_create_gc(conn, pixmap_gc, pixmap, 0, 0);
 
     /* Grab the keyboard to get all input */
@@ -512,7 +520,7 @@ int main(int argc, char *argv[]) {
 
             case XCB_CONFIGURE_NOTIFY: {
                 xcb_configure_notify_event_t *configure_notify = (xcb_configure_notify_event_t *)event;
-                rect = (xcb_rectangle_t) {
+                rect = (xcb_rectangle_t){
                     configure_notify->x,
                     configure_notify->y,
                     configure_notify->width,
