@@ -18,7 +18,6 @@
  */
 Con *con_new_skeleton(Con *parent, i3Window *window);
 
-
 /* A wrapper for con_new_skeleton, to retain the old con_new behaviour
  *
  */
@@ -173,6 +172,18 @@ void con_fix_percent(Con *con);
  */
 void con_toggle_fullscreen(Con *con, int fullscreen_mode);
 
+/**
+ * Enables fullscreen mode for the given container, if necessary.
+ *
+ */
+void con_enable_fullscreen(Con *con, fullscreen_mode_t fullscreen_mode);
+
+/**
+ * Disables fullscreen mode for the given container, if necessary.
+ *
+ */
+void con_disable_fullscreen(Con *con);
+
 /**
  * Moves the given container to the currently focused container on the given
  * workspace.
@@ -345,3 +356,9 @@ void con_set_urgency(Con *con, bool urgent);
  *
  */
 char *con_get_tree_representation(Con *con);
+
+/**
+ * force parent split containers to be redrawn
+ *
+ */
+void con_force_split_parents_redraw(Con *con);
