@@ -68,25 +68,25 @@ void workspace_show_by_name(const char *num);
  * Returns the next workspace.
  *
  */
-Con* workspace_next(void);
+Con *workspace_next(void);
 
 /**
  * Returns the previous workspace.
  *
  */
-Con* workspace_prev(void);
+Con *workspace_prev(void);
 
 /**
  * Returns the next workspace on the same output
  *
  */
-Con* workspace_next_on_output(void);
+Con *workspace_next_on_output(void);
 
 /**
  * Returns the previous workspace on the same output
  *
  */
-Con* workspace_prev_on_output(void);
+Con *workspace_prev_on_output(void);
 
 /**
  * Focuses the previously focused workspace.
@@ -100,7 +100,6 @@ void workspace_back_and_forth(void);
  */
 Con *workspace_back_and_forth_get(void);
 
-
 #if 0
 /**
  * Assigns the given workspace to the given screen by correctly updating its
@@ -180,3 +179,10 @@ Con *workspace_attach_to(Con *ws);
  * The container inherits the layout from the workspace.
  */
 Con *workspace_encapsulate(Con *ws);
+
+/**
+ * Move the given workspace to the specified output.
+ * This returns true if and only if moving the workspace was successful.
+ *
+ */
+bool workspace_move_to_output(Con *ws, char *output);
