@@ -11,7 +11,8 @@
  */
 #include "all.h"
 
-typedef enum { BEFORE, AFTER } position_t;
+typedef enum { BEFORE,
+               AFTER } position_t;
 
 /*
  * This function detaches 'con' from its parent and inserts it either before or
@@ -127,18 +128,21 @@ static void move_to_output_directed(Con *con, direction_t direction) {
 
     tree_flatten(croot);
 
-    ipc_send_workspace_focus_event(ws, old_ws);
+    ipc_send_workspace_event("focus", ws, old_ws);
 }
 
 /*
- * Moves the current container in the given direction (D_LEFT, D_RIGHT,
+ * Moves the given container in the given direction (D_LEFT, D_RIGHT,
  * D_UP, D_DOWN).
  *
  */
-void tree_move(int direction) {
+void tree_move(Con *con, int direction) {
+    position_t position;
+    Con *target;
+
     DLOG("Moving in direction %d\n", direction);
+
     /* 1: get the first parent with the same orientation */
-    Con *con = focused;
 
     if (con->type == CT_WORKSPACE) {
         DLOG("Not moving workspace\n");
@@ -180,21 +184,28 @@ void tree_move(int direction) {
         if (same_orientation == con->parent) {
             DLOG("We are in the same container\n");
             Con *swap;
-            if ((swap = (direction == D_LEFT || direction == D_UP ?
-                          TAILQ_PREV(con, nodes_head, nodes) :
-                          TAILQ_NEXT(con, nodes)))) {
+            if ((swap = (direction == D_LEFT || direction == D_UP ? TAILQ_PREV(con, nodes_head, nodes) : TAILQ_NEXT(con, nodes)))) {
                 if (!con_is_leaf(swap)) {
-                    insert_con_into(con, con_descend_focused(swap), AFTER);
+                    DLOG("Moving into our bordering branch\n");
+                    target = con_descend_direction(swap, direction);
+                    position = (con_orientation(target->parent) != o ||
+                                        direction == D_UP ||
+                                        direction == D_LEFT
+                                    ? AFTER
+                                    : BEFORE);
+                    insert_con_into(con, target, position);
                     goto end;
                 }
                 if (direction == D_LEFT || direction == D_UP)
                     TAILQ_SWAP(swap, con, &(swap->parent->nodes_head), nodes);
-                else TAILQ_SWAP(con, swap, &(swap->parent->nodes_head), nodes);
+                else
+                    TAILQ_SWAP(con, swap, &(swap->parent->nodes_head), nodes);
 
                 TAILQ_REMOVE(&(con->parent->focus_head), con, focused);
                 TAILQ_INSERT_HEAD(&(swap->parent->focus_head), con, focused);
 
                 DLOG("Swapped.\n");
+                ipc_send_window_event("move", con);
                 return;
             }
 
@@ -202,6 +213,7 @@ void tree_move(int direction) {
                 /*  If we couldn't find a place to move it on this workspace,
                  *  try to move it to a workspace on a different output */
                 move_to_output_directed(con, direction);
+                ipc_send_window_event("move", con);
                 return;
             }
 
@@ -225,22 +237,23 @@ void tree_move(int direction) {
     }
 
     DLOG("above = %p\n", above);
-    Con *next;
-    position_t position;
-    if (direction == D_UP || direction == D_LEFT) {
-        position = BEFORE;
-        next = TAILQ_PREV(above, nodes_head, nodes);
-    } else {
-        position = AFTER;
-        next = TAILQ_NEXT(above, nodes);
-    }
 
-    /* special case: there is a split container in the direction we are moving
-     * to, so descend and append */
-    if (next && !con_is_leaf(next))
-        insert_con_into(con, con_descend_focused(next), AFTER);
-    else
+    Con *next = (direction == D_UP || direction == D_LEFT ? TAILQ_PREV(above, nodes_head, nodes) : TAILQ_NEXT(above, nodes));
+
+    if (next && !con_is_leaf(next)) {
+        DLOG("Moving into the bordering branch of our adjacent container\n");
+        target = con_descend_direction(next, direction);
+        position = (con_orientation(target->parent) != o ||
+                            direction == D_UP ||
+                            direction == D_LEFT
+                        ? AFTER
+                        : BEFORE);
+        insert_con_into(con, target, position);
+    } else {
+        DLOG("Moving into container above\n");
+        position = (direction == D_UP || direction == D_LEFT ? BEFORE : AFTER);
         insert_con_into(con, above, position);
+    }
 
 end:
     /* We need to call con_focus() to fix the focus stack "above" the container
@@ -252,4 +265,5 @@ end:
     FREE(con->deco_render_params);
 
     tree_flatten(croot);
+    ipc_send_window_event("move", con);
 }
